#!/bin/bash
#
# Usage: install.sh <imagefile>
#
# VERSION=16
# CHANGES="add new line to update.log from here"

BEROCONF=/usr/fallback/beroconf
file=$1
rebootflag=$2
image_dir=/tmp/images


# functions #

function log {
	echo "[${1}_install] $(date): ${2}"
}

function fix_old_image_dir {

	mount -o remount,rw /

	if [ -d /mnt/images ]; then
		rm -rf /mnt/images
	fi

	ln -s /tmp /mnt/images

	mount -o remount,ro /
}

function tmp_cleanup {

	if [ -f /tmp/VERSION ]; then
		rm -f /tmp/VERSION
	fi

	if [ -f /tmp/content.txt ]; then
		rm -f /tmp/content.txt
	fi

	if [ -f /tmp/compare-content.txt ]; then
		rm -f /tmp/compare-content.txt
	fi
}

function cron_daemon {

	/etc/init.d/S45cron ${1} > /dev/null
}


# main #

if [ ! -d ${image_dir} ]; then
	mkdir -p ${image_dir}
fi

if [ -z "${file}" ] ; then
	log "common" "FAIL: No filename given, leaving."
	exit 1
fi

if [ ! -f "${image_dir}/${file}" ] ; then
	log "common" "FAIL: File ${image_dir}/${file} not found, leaving."
	exit 1
fi

log "common" "getting VERSION file"
cd /tmp && tar zxf ${image_dir}/${file} VERSION
. /tmp/VERSION

if [ ! -L /mnt/images ]; then
	log "common" "INFO: /mnt/images is not a symlink, fixing that."
	fix_old_image_dir
fi

log "common" "installing file: ${file}, type: ${TYPE}"
PRFX=${TYPE}

case "${TYPE}" in
	fpga)
		log "${PRFX}" "unpacking fpga image"
		cd /tmp && tar zxf ${image_dir}/${file} fpga.img

		/sbin/insmod /usr/local/modules/bfpga.ko
		if [ -f /tmp/fpga.img ] ; then
			log "${PRFX}" "updating version file"
			mv /tmp/VERSION /usr/conf/VERSION.fpga

			VERSION_INSTALLED="$(cat /proc/FPGA_VERSION)"
			VERSION_TOINSTALL="$(cat /usr/conf/VERSION.${TYPE})"
			echo "[${TYPE}] $(date): ${VERSION_INSTALLED} => ${VERSION_TOINSTALL}" >> /usr/conf/update.log

			log "${PRFX}" "installing fpga image, please stand by."
			log "${PRFX}" "This process may take 3 minutes. Do NOT turn off bero*fix!"
			dd if=/tmp/fpga.img of=/dev/bfpga bs=512k count=1
		fi
	;;
	firmware)
		if [ -f /usr/conf/VERSION.${TYPE} ]; then
			VERSION_INSTALLED="$(cat /usr/conf/VERSION.${TYPE})"
		else
			VERSION_INSTALLED="unknown"
		fi

		log "${PRFX}" "unpacking firmware"
		cd /tmp && tar zxf ${image_dir}/${file} firmware.axf

		log "${PRFX}" "updating firmware"
		dd if=/tmp/firmware.axf of=/dev/mtdblock3

		log "${PRFX}" "removing temporary file"
		rm /tmp/firmware.axf

		log "${PRFX}" "updating version file"
		mv /tmp/VERSION /usr/conf/VERSION.firmware

		VERSION_TOINSTALL="$(cat /usr/conf/VERSION.${TYPE})"
		echo "[${TYPE}] $(date): ${VERSION_INSTALLED} => ${VERSION_TOINSTALL}" >> /usr/conf/update.log
	;;
	kernel)
		if [ -f /usr/conf/VERSION.${TYPE} ]; then
			VERSION_INSTALLED="$(cat /usr/conf/VERSION.${TYPE})"
		else
			VERSION_INSTALLED="unknown"
		fi

		log "${PRFX}" "unpacking kernel"
		cd /tmp && tar zxf ${image_dir}/${file} zImage

		log "${PRFX}" "updating kernel"
		dd if=/tmp/zImage of=/dev/mtdblock4

		log "${PRFX}" "removing temporary file"
		rm /tmp/zImage

		log "${PRFX}" "updating version file"
		mv /tmp/VERSION /usr/conf/VERSION.kernel

		VERSION_TOINSTALL="$(cat /usr/conf/VERSION.${TYPE})"
		echo "[${TYPE}] $(date): ${VERSION_INSTALLED} => ${VERSION_TOINSTALL}" >> /usr/conf/update.log
	;;
	rootfs)
		if [ -f /VERSION ]; then
			VERSION_INSTALLED=$(expr match "$(cat /VERSION)" ".*ROOTFS=\([0-9]\)")
		else
			VERSION_INSTALLED="unknown"
		fi

		log "${PRFX}" "stopping services"
		/etc/init.d/S90app stop
		/etc/init.d/S60httpd stop

		log "${PRFX}" "loading berofix module"
		insmod /usr/local/modules/berofix.ko

		if [ -f /etc/ssh_host_key ]
		then
			log "${PRFX}" "saving ssh keys to conf partition"
			[ ! -d /usr/conf/ssh ] && mkdir /usr/conf/ssh
			cp -a /etc/ssh_host* /usr/conf/ssh/
		fi

		log "${PRFX}" "extracting new rootfs"
		rm -rf /tmp/newroot
		mkdir /tmp/newroot || exit 1
		cd /tmp/newroot
		tar zxf ${image_dir}/${file} root.jffs2 || exit 1

		log "${PRFX}" "writing new rootfs to flash memory"
		dd if=root.jffs2 of=/dev/mtdblock5

		VERSION_TOINSTALL=$(expr match "$(cat /VERSION)" ".*ROOTFS=\([0-9]\)")
		echo "[${TYPE}] $(date): ${VERSION_INSTALLED} => ${VERSION_TOINSTALL}" >> /usr/conf/update.log

		log "${PRFX}" "rebooting"
		dd if=/proc/RESET of=/dev/null
	;;
	userappfs)
		USERAPP_ROOT=/apps
		USERAPP_HOME=/home/admin

		logmode=$(${BEROCONF} get root userapp-logmode | grep -v failed)
		if [ "${logmode}" = "1" ]; then
			log "${PRFX}" "logmode enabled, aborting install."
			exit 1
		fi

		if [ -z ${NAME} ]; then
			log "${PRFX}" "name of this UserApp is empty, leaving."
			exit 1
		fi

		log "${PRFX}" "cleaning up /tmp."
		tmp_cleanup

		log "${PRFX}" "installing ${NAME}"
		log "${PRFX}" "description: ${DESCRIPTION}"

		log "${PRFX}" "checking archive integrity"
		cd /tmp
		tar zxf ${image_dir}/${file} content.txt 2> /dev/null
		if [ ! -f /tmp/content.txt ]; then
			log "${PRFX}" "FAIL: Archive not compatible!"
			exit 1
		fi
		tar tzf ${image_dir}/${file}  > /tmp/compare-content.txt
		for i in $(cat /tmp/content.txt | sed "s/\/$//" )  ; do
			if ! grep "${i}" compare-content.txt > /dev/null ; then
				log "${PRFX}" "FAIL: File '${i}' missing in archive, leaving."
				exit 1
			fi
		done

		log "${PRFX}" "checking versions"
#		# check kernel
#		if [ -f /usr/conf/VERSION.kernel ] ; then
#			KERNEL=$(expr match "$(cat /usr/conf/VERSION.kernel)" ".*VERSION=\([0-9]*\)")
#		else
#			KERNEL=1
#		fi
#		if [ ${NEED_KERNEL} -gt ${KERNEL} ] ; then
#			log "${PRFX}" "FAIL: Please update kernel to version ${NEED_KERNEL} and try again!"
#			exit 1
#		fi
#
#		# check rootfs
#		if [ -f /VERSION ] ; then
#			KERNEL=$(expr match "$(cat /VERSION)" ".*ROOTFS=\([0-9]*\)")
#		else
#			ROOTFS=1
#		fi
#		if [ ${NEED_ROOTFS} -gt ${ROOTFS} ] ; then
#			log "${PRFX}" "FAIL: Please update rootfs to version ${NEED_ROOTFS} and try again!"
#			exit 1
#		fi
#
#		# check firmware
#		if [ -f /usr/conf/VERSION.firmware ]; then
#			FIRMWARE=$(expr match "$(cat /usr/conf/VERSION.firmware)" ".*VERSION=\([0-9]*\)")
#		else
#			FIRMWARE=1
#		fi
#		if [ ${NEED_FIRMWARE} -gt ${FIRMWARE} ]; then
#			log "${PRFX}" "FAIL: Please update firmware to version ${NEED_FIRMWARE} and try again!"
#			exit 1
#		fi

		# check appfs
		if [ -f /usr/local/VERSION ]; then
			APPFS_FIL="$(cat /usr/local/VERSION)"
			APPFS=$(expr match "${APPFS_FIL}" ".*APPFS=\([0-9]*\)")
			unset APPFS_FIL
		else
			APPFS=1
		fi
		if [ ${NEED_APPFS} -gt ${APPFS} ]; then
			log "${PRFX}" "FAIL: Please update AppFS to version ${NEED_APPFS} and try again!"
			exit 1
		fi

		# if this is not an upgrade, check if there is enough space left
		if [ ! -d /apps/${NAME} ]; then

			if [ -z "${SIZE}" ]; then
				let SIZE="${SLOTS}*1024"
			else
				let SIZE="${SIZE}/1024"
			fi

			SIZE_FREE=$(df | grep ${USERAPP_HOME} | awk '{ print $4; }')

			if [ ${SIZE} -ge ${SIZE_FREE} ]; then
				log "${PRFX}" "FAIL: Installing this app will exceed free disk-space (app-size: ${SIZE} KBytes, free: ${SIZE_FREE} KBytes)."
				exit 1
			fi

			VERSION_INSTALLED="${NAME} none"
		else
			VERSION_INSTALLED=$(/usr/bin/expr match "$(/bin/cat /apps/${NAME}/VERSION)" ".*VERSION=\([0-9]\)")
			VERSION_INSTALLED="${NAME} ${VERSION_INSTALLED}"
		fi

		cron_daemon stop

		log "${PRFX}" "stopping services"
		/etc/init.d/S91userapp stop > /dev/null

		log "${PRFX}" "remounting userapp partition (rw)"
		/bin/mount -oremount,rw ${USERAPP_HOME} || exit 1

		if [ -d ${USERAPP_ROOT}/${NAME} ]; then
			log "${PRFX}" "executing userapp pre_remove script"
			[ -x ${USERAPP_HOME}/setup/pre_remove.sh ] && ${USERAPP_HOME}/setup/pre_remove.sh ${NAME} update

			log "${PRFX}" "cleaning up ${NAME}"
			rm -rf ${USERAPP_ROOT}/${NAME}
		fi

		log "${PRFX}" "extracting new image (${file})"
		cd ${USERAPP_ROOT} && tar zxf ${image_dir}/${file} || exit 1

		log "${PRFX}" "executing post_install script"
		[ -x ${USERAPP_HOME}/setup/post_install.sh ] && ${USERAPP_HOME}/setup/post_install.sh ${NAME}

		log "${PRFX}" "updating PATH and LD_LIBRARY_PATH variables in .bash_profile"
		for i in $(ls /apps/); do
			path_str_bin="${path_str_bin}:/apps/${i}/bin"
			path_str_lib="${path_str_lib}:/apps/${i}/lib"
		done
		path_str_bin=$(echo ${path_str_bin} | sed -e 's/\([[\/.*]\|\]\)/\\&/g')
		path_str_lib=$(echo ${path_str_lib} | sed -e 's/\([[\/.*]\|\]\)/\\&/g')
		sed "s/APPS_PATHS/${path_str_bin}/g" /home/admin/bash_profile_skel > /tmp/admin_bash_profile.tmp
		sed "s/LIBS_PATHS/${path_str_lib}/g" /tmp/admin_bash_profile.tmp > /home/admin/.bash_profile
		rm -f /tmp/admin_bash_profile.tmp

		log "${PRFX}" "setting permissions"
		chown -R admin:admin ${USERAPP_ROOT}/${NAME}

		log "${PRFX}" "waiting and syncing"
		sleep 1; sync; sync; sleep 1

		log "${PRFX}" "remounting partition (ro)"
		/bin/mount -oremount,ro ${USERAPP_HOME} || exit 1

		log "${PRFX}" "cleaning up /tmp"
		tmp_cleanup

		log "${PRFX}" "starting services"
		/etc/init.d/S91userapp start > /dev/null

		cron_daemon start

		VERSION_TOINSTALL=$(expr match "$(cat /apps/${NAME}/VERSION)" ".*VERSION=\([0-9]\)")
		echo "[${TYPE}] $(date): ${VERSION_INSTALLED} => ${VERSION_TOINSTALL}" >> /usr/conf/update.log
		;;
	*)
		DOWNGRADE_IND=$(echo ${file} | grep "downgrade")
		if [ -z "${DOWNGRADE_IND}" ]; then
			log "${PRFX}" "checking for archive integrity"
			cd /tmp && tar zxf ${image_dir}/${file} content.txt 2>/dev/null
			if [ ! -f /tmp/content.txt ]; then
				log "${PRFX}" "FAIL: archive not compatible!"
				exit 1
			fi
			tar tzf ${image_dir}/${file} > /tmp/compare-content.txt
			for i in $(cat /tmp/content.txt | sed "s/\/$//" ); do
				if ! grep "${i}" compare-content.txt > /dev/null; then
					log "${PRFX}" "FAIL: File ${i} missing in archive, leaving!"
					exit 1
				fi
			done
		fi

		rm -rf /tmp/content.txt
                rm -rf /tmp/compare-content.txt

		log "${PRFX}" "checking versions"
		if [ -f /usr/conf/VERSION.kernel ]; then
			KERNEL=$(expr match "$(cat /usr/conf/VERSION.kernel)" ".*VERSION=\([0-9]*\)")
		else
			KERNEL=1
		fi

		if [ -f /usr/conf/VERSION.firmware ]; then
			FIRMWARE=$(expr match "$(cat /usr/conf/VERSION.firmware)" ".*VERSION=\([0-9]*\)")
		else
			FIRMWARE=1
		fi

		if [ -f /VERSION ]; then
			ROOTFS=$(expr match "$(cat /VERSION)" ".*ROOTFS=\([0-9]*\)")
		else
			ROOTFS=1
		fi

		if [ "${NEED_KERNEL}" -gt "${KERNEL}" ]; then
			log "${PRFX}" "FAIL: Please update kernel to version ${NEED_KERNEL} and try again!"
			exit 1
		fi
		if [ "${NEED_FIRMWARE}" -gt "${FIRMWARE}" ]; then
			log "${PRFX}" "FAIL: Please update firmware to version ${NEED_FIRMWARE} and try again!"
			exit 1
		fi
		if [ "${NEED_ROOTFS}" -gt "${ROOTFS}" ]; then
			log "${PRFX}" "FAIL: Please update rootfs to version ${NEED_ROOTFS} and try again!"
			exit 1
		fi

		VERSION_INSTALLED="$(cat /usr/local/FILENAME)"

		cron_daemon stop

                log "${PRFX}" "stopping UserAppFS services"
                /etc/init.d/S91userapp stop > /dev/null

		log "${PRFX}" "stopping AppFS services"
		/etc/init.d/S90app stop

		log "${PRFX}" "remounting AppFS partition (rw)"
		mount -o remount,rw /usr/local || exit 1

		log "${PRFX}" "executing pre_remove script"
		[ -x /usr/local/setup/pre_remove.sh ] && /usr/local/setup/pre_remove.sh

		log "${PRFX}" "removing old version file"
		[ -f /usr/local/FILENAME ] && rm -f /usr/local/FILENAME

		log "${PRFX}" "cleaning up AppFS partition"
		cd /usr/local && ls | grep -v lost | xargs rm -rf

		log "${PRFX}" "extracting new AppFS image"
		cd /usr/local && tar zxf ${image_dir}/${file} || exit 1

		log "${PRFX}" "setting permissions"
		chown -R root:root /usr/local/*

		log "${PRFX}" "writing new version file"
		echo "${file}" > /usr/local/FILENAME

		if [ ! -z "${DOWNGRADE_IND}" ]; then
			log "${PRFX}" "cleaning up downgrade-incompatible configuration files"
			rm -f /usr/conf/isgw.gsm
		fi

		log "${PRFX}" "waiting and syncing"
		sleep 1; sync; sync; sleep 1

                log "${PRFX}" "removing uploaded AppFS-image"
                rm -f ${image_dir}/${file}
		if [ -f /home/admin/${file} ]; then
			mount -oremount,rw /home/admin
			rm -f /home/admin/${file}
			mount -oremount,ro /home/admin
		fi

		log "${PRFX}" "executing post_install script"
		[ -x /usr/local/setup/post_install.sh ] && /usr/local/setup/post_install.sh

		log "${PRFX}" "remounting AppFS partition (ro)"
		cd / && mount -o remount,ro /usr/local

                sync

		${BEROCONF} set root app-status running
		${BEROCONF} set root app-image $(cat /usr/local/FILENAME)

#                if [ "$rebootflag" == "" ] ; then
#			log "${PRFX}" "rebooting"
#			/sbin/reboot
#			exit 1
#                fi

		log "${PRFX}" "starting AppFS services"
		for i in /usr/local/init/S??* ; do
		        # Ignore dangling symlinks (if any).
		        [ ! -f "$i" ] && continue
		        if [ "$i" = "/usr/local/init/S99upgrade-firmware" ] || [ "$i" = "/usr/local/init/S02update-conf" ]; then
                       	log "${PRFX}" "skipping service: ${i}"
	                        continue
        		fi
        		log "${PRFX}" "starting service: ${i}"
	        	${i} start
		done

                log "${PRFX}" "starting UserAppFS services"
                /etc/init.d/S91userapp start > /dev/null

		cron_daemon start

		VERSION_TOINSTALL="$(cat /usr/local/FILENAME)"
		echo "[${TYPE}] $(date): ${VERSION_INSTALLED} => ${VERSION_TOINSTALL}" >> /usr/conf/update.log
	;;
esac

log "${PRFX}" "done"

exit 0
